/**
 * CheckUpdate Setting...
 */
app.config(['$stateProvider', '$urlRouterProvider', '$httpProvider', function ($stateProvider, $urlRouterProvider, $httpProvider) {
    $httpProvider.interceptors.push(['UpdateService', function (UpdateService) {
        return {
            'request': function (config) {
            	if (typeof config == 'undefined' || config.url.indexOf('https') >= 0) return config;
//            	config.url = window.Manifest.root + config.url;

                if (UpdateService.isFileCached(config.url)) {

                    var url = UpdateService.getCachedUrl(config.url);
                    config.url = url;
                }

                return config;
            },
            'response': function (response) {

                return response;
            }
        };
    }]); /* end of http UpdateService */
}]); /* end of config */


angular.module('update.services', [])
	/**
	 * Upgrade Functions
	 */
	.factory('UpdateService', ['$log', 'ConfigService', '$q', function ($log, ConfigService, $q) {
    
		var fs = new CordovaPromiseFS({ persistent: typeof cordova !== 'undefined', Promise: $q });
		console.log(ConfigService.getHost());
	    var loader = new CordovaAppLoader({
	        fs: fs,
	        serverRoot: ConfigService.getHost(),
	        localRoot: 'data/.com.korloy.kts/',
	        cacheBuster: true, // make sure we're not downloading cached files.
	        checkTimeout: 1000, // timeout for the "check" function - when you loose internet connection
	        mode: 'hash', // mirror, hash
	    });
    
	    var service = {
		cacheDelete : function(){
			loader.clear();
//			location.reload();
		},
	        // Check for new updates on js and css files
	        check: function () {
	
	        	var host = ConfigService.getHost();
	        	var cacheBuster = ConfigService.getManifest()+"?"+Math.random();
	        	
	        	console.log("check url >> " + host + cacheBuster);
	        	
	        	var defer = $q.defer();
	        	loader.check(host + cacheBuster).then(function (updateAvailable) {
	                console.log("Update available:"+updateAvailable);
	                if (updateAvailable) {
	                    defer.resolve(updateAvailable);
	                }
	                else {
	                    defer.reject(updateAvailable);
	                }
	            }, function(error) {
	            	defer.reject(error);
	            });
	            return defer.promise;
	        },
	        // Download new js/css files 
	        download: function (onprogress) {
	            var defer = $q.defer();
	
	            loader.download(onprogress).then(function (manifest) {
	                console.log("Download active!");
	                defer.resolve(manifest);
	            }, function (error) {
	                console.log("Download Error:");
	                defer.reject(error);
	            });
	            return defer.promise;
	        },
	        // Update the local files with a new version just downloaded
	        update: function (reload) {
	            console.log("update files--------------");
	            return loader.update(reload);
	        },
	        // Check wether the HTML file is cached
	        isFileCached: function (file) {
	            if (angular.isDefined(loader.cache)) {
	                return loader.cache.isCached(file);
	            }
	            return false;
	        },
	        // returns the cached HTML file as a url for HTTP interceptor
	        getCachedUrl : function (url) {
	            if(angular.isDefined(loader.cache)) {
	                return loader.cache.get(url);
	            }
	            return url;
	        }
	    };
	
	    return service;
	}]) /* end of UpdateService */

	.factory('ConfigService', [function() {
		var hostURL = "http://kts.korloy.com/app/kts1-0-2/";
	    var manifest = "manifest.json";
	    
	    var service = {
	        getHost : function () {
	        	console.log("ConfigService hostURL >> " + hostURL);
	            return hostURL;
	        },
	    
	        getManifest : function () {
	        	console.log("ConfigService manifest >> " + manifest);
	        	return manifest;
	        }
	    }
	    return service;
	}]); /* end of ConfigService */
